@@ -12,7 +12,6 @@
 #include "loader.h"
 #include "str.h"
 #include "sway.h"
-#include "text.h"
 #include "ui.h"
 #include "viewer.h"
 
@@ -21,6 +20,7 @@
 #include <limits.h>
 #include <poll.h>
 #include <pthread.h>
+#include <signal.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/wait.h>
@@ -64,6 +64,9 @@ struct application {
     pthread_mutex_t events_lock; ///< Event queue lock
     int event_signal;            ///< Queue change notification
 
+    struct action_seq sigusr1; ///< Actions applied by USR1 signal
+    struct action_seq sigusr2; ///< Actions applied by USR2 signal
+
     event_handler ehandler; ///< Event handler for the current mode
     struct rect window;     ///< Preferable window position and size
     char* app_id;           ///< Application id (app_id name)
@@ -177,6 +180,71 @@ static void append_event(const struct event* event)
     notification_raise(ctx.event_signal);
 }
 
+/**
+ * Apply action.
+ * @param action pointer to the action being performed
+ */
+static void apply_action(const struct action* action)
+{
+    struct event event;
+
+    switch (action->type) {
+        case action_info:
+            info_switch(action->params);
+            app_redraw();
+            break;
+        case action_status:
+            info_update(info_status, "%s", action->params);
+            app_redraw();
+            break;
+        case action_fullscreen:
+            ui_toggle_fullscreen();
+            break;
+        case action_help:
+            info_switch_help();
+            app_redraw();
+            break;
+        case action_exit:
+            if (info_help_active()) {
+                info_switch_help(); // remove help overlay
+                app_redraw();
+            } else {
+                app_exit(0);
+            }
+            break;
+        default:
+            // not a general action, add to event queue
+            event.type = event_action;
+            event.param.action = action;
+            append_event(&event);
+            break;
+    }
+}
+
+/**
+ * POSIX Signal handler.
+ * @param signum signal number
+ */
+static void on_signal(int signum)
+{
+    const struct action_seq* sigact;
+
+    switch (signum) {
+        case SIGUSR1:
+            sigact = &ctx.sigusr1;
+            break;
+        case SIGUSR2:
+            sigact = &ctx.sigusr2;
+            break;
+        default:
+            return;
+    }
+
+    for (size_t i = 0; i < sigact->num; ++i) {
+        apply_action(&sigact->sequence[i]);
+    }
+}
+
 /**
  * Load first (initial) image.
  * @param index initial index of image in the image list
@@ -283,6 +351,14 @@ static enum config_status load_config(const char* key, const char* value)
                 status = cfgst_ok;
             }
         }
+    } else if (strcmp(key, APP_CFG_SIGUSR1) == 0) {
+        if (action_create(value, &ctx.sigusr1)) {
+            status = cfgst_ok;
+        }
+    } else if (strcmp(key, APP_CFG_SIGUSR2) == 0) {
+        if (action_create(value, &ctx.sigusr2)) {
+            status = cfgst_ok;
+        }
     } else if (strcmp(key, APP_CFG_APP_ID) == 0) {
         str_dup(value, &ctx.app_id);
         status = cfgst_ok;
@@ -301,11 +377,13 @@ void app_create(void)
     ctx.window.height = SIZE_FROM_PARENT;
     ctx.ehandler = viewer_handle;
 
+    action_create("reload", &ctx.sigusr1);
+    action_create("next_file", &ctx.sigusr2);
+
     font_create();
     image_list_create();
     info_create();
     keybind_create();
-    text_create();
     viewer_create();
     gallery_create();
 
@@ -315,9 +393,9 @@ void app_create(void)
 
 void app_destroy(void)
 {
+    loader_destroy();
     gallery_destroy();
     viewer_destroy();
-    loader_destroy();
     ui_destroy();
     image_list_destroy();
     info_destroy();
@@ -332,18 +410,25 @@ void app_destroy(void)
     while (ctx.events) {
         struct event_entry* entry = ctx.events;
         ctx.events = ctx.events->next;
+        if (entry->event.type == event_load) {
+            image_free(entry->event.param.load.image);
+        }
         free(entry);
     }
     if (ctx.event_signal != -1) {
         notification_free(ctx.event_signal);
     }
     pthread_mutex_destroy(&ctx.events_lock);
+
+    action_free(&ctx.sigusr1);
+    action_free(&ctx.sigusr2);
 }
 
 bool app_init(const char** sources, size_t num)
 {
     bool force_load = false;
     struct image* first_image;
+    struct sigaction sigact;
 
     // compose image list
     if (num == 0) {
@@ -380,9 +465,8 @@ bool app_init(const char** sources, size_t num)
     }
     if (ctx.window.width == SIZE_FULLSCREEN) {
         ui_toggle_fullscreen();
-    } else if (first_image &&
-               (ctx.window.width == SIZE_FROM_IMAGE ||
-                ctx.window.width == SIZE_FROM_PARENT)) {
+    } else if (ctx.window.width == SIZE_FROM_IMAGE ||
+               ctx.window.width == SIZE_FROM_PARENT) {
         // fixup window size form the first image
         const struct pixmap* pm = &first_image->frames[0].pm;
         ctx.window.width = pm->width;
@@ -420,6 +504,13 @@ bool app_init(const char** sources, size_t num)
                                                   : APP_MODE_GALLERY);
     }
 
+    // set signal handler
+    sigact.sa_handler = on_signal;
+    sigemptyset(&sigact.sa_mask);
+    sigact.sa_flags = 0;
+    sigaction(SIGUSR1, &sigact, NULL);
+    sigaction(SIGUSR2, &sigact, NULL);
+
     return true;
 }
 
@@ -457,10 +548,12 @@ bool app_run(void)
         ui_event_prepare();
 
         // poll events
-        if (poll(fds, ctx.wfds_num, -1) <= 0) {
-            perror("Error polling events");
-            ctx.state = loop_error;
-            break;
+        if (poll(fds, ctx.wfds_num, -1) < 0) {
+            if (errno != EINTR) {
+                perror("Error polling events");
+                ctx.state = loop_error;
+                break;
+            }
         }
 
         // call handlers for each active event
@@ -504,6 +597,9 @@ void app_switch_mode(size_t index)
     if (info_enabled()) {
         info_switch(info_mode);
     }
+    if (info_help_active()) {
+        info_switch_help();
+    }
 
     app_redraw();
 }
@@ -515,8 +611,10 @@ bool app_is_viewer(void)
 
 void app_reload(void)
 {
+    static const struct action action = { .type = action_reload };
     const struct event event = {
-        .type = event_reload,
+        .type = event_action,
+        .param.action = &action,
     };
     append_event(&event);
 }
@@ -558,12 +656,20 @@ void app_on_resize(void)
 
 void app_on_keyboard(xkb_keysym_t key, uint8_t mods)
 {
-    const struct event event = {
-        .type = event_keypress,
-        .param.keypress.key = key,
-        .param.keypress.mods = mods,
-    };
-    append_event(&event);
+    const struct keybind* kb = keybind_find(key, mods);
+
+    if (kb) {
+        for (size_t i = 0; i < kb->actions.num; ++i) {
+            apply_action(&kb->actions.sequence[i]);
+        }
+    } else {
+        char* name = keybind_name(key, mods);
+        if (name) {
+            info_update(info_status, "Key %s is not bound", name);
+            free(name);
+            app_redraw();
+        }
+    }
 }
 
 void app_on_drag(int dx, int dy)
