@@ -9,7 +9,6 @@
 #include "info.h"
 #include "loader.h"
 #include "str.h"
-#include "text.h"
 #include "ui.h"
 
 #include <stdlib.h>
@@ -34,6 +33,8 @@ struct gallery {
     argb_t clr_window;     ///< Window background
     argb_t clr_background; ///< Tile background
     argb_t clr_select;     ///< Selected tile background
+    argb_t clr_border;     ///< Selected tile border
+    argb_t clr_shadow;     ///< Selected tile shadow
 
     size_t top;      ///< Index of the first displayed image
     size_t selected; ///< Index of the selected image
@@ -142,8 +143,6 @@ static void draw_thumbnail(struct pixmap* window, ssize_t x, ssize_t y,
         // currently selected item
         const size_t thumb_size = THUMB_SELECTED_SCALE * ctx.thumb_size;
         const size_t thumb_offset = (thumb_size - ctx.thumb_size) / 2;
-        const size_t shadow_width = max(1, thumb_size / 15);
-        const size_t alpha_step = 0xff / shadow_width;
 
         x = max(0, x - (ssize_t)thumb_offset);
         y = max(0, y - (ssize_t)thumb_offset);
@@ -154,8 +153,10 @@ static void draw_thumbnail(struct pixmap* window, ssize_t x, ssize_t y,
         pixmap_fill(window, x, y, thumb_size, thumb_size, ctx.clr_select);
 
         if (thumb) {
-            const ssize_t tx = x + ctx.thumb_size / 2 - thumb->width / 2;
-            const ssize_t ty = y + ctx.thumb_size / 2 - thumb->height / 2;
+            const ssize_t thumb_w = thumb->width * THUMB_SELECTED_SCALE;
+            const ssize_t thumb_h = thumb->height * THUMB_SELECTED_SCALE;
+            const ssize_t tx = x + thumb_size / 2 - thumb_w / 2;
+            const ssize_t ty = y + thumb_size / 2 - thumb_h / 2;
             if (ctx.thumb_aa) {
                 pixmap_scale_bicubic(thumb, window, tx, ty,
                                      THUMB_SELECTED_SCALE, image->alpha);
@@ -166,23 +167,33 @@ static void draw_thumbnail(struct pixmap* window, ssize_t x, ssize_t y,
         }
 
         // shadow
-        for (size_t i = 0; i < shadow_width; ++i) {
-            const ssize_t lx = i + x + thumb_size;
-            const ssize_t ly = y + shadow_width;
-            const ssize_t lh = thumb_size - (shadow_width - i);
-            const argb_t color = ARGB_SET_A(0xff - i * alpha_step);
-            pixmap_vline(window, lx, ly, lh, color);
-        }
-        for (size_t i = 0; i < shadow_width; ++i) {
-            const ssize_t lx = x + shadow_width;
-            const ssize_t ly = y + thumb_size + i;
-            const ssize_t lw = thumb_size - (shadow_width - i) + 1;
-            const argb_t color = ARGB_SET_A(0xff - i * alpha_step);
-            pixmap_hline(window, lx, ly, lw, color);
+        if (ARGB_GET_A(ctx.clr_shadow)) {
+            const argb_t base = ctx.clr_shadow & 0x00ffffff;
+            const uint8_t alpha = ARGB_GET_A(ctx.clr_shadow);
+            const size_t width =
+                max(1, (double)thumb_size / 15.0 * ((double)alpha / 255.0));
+            const size_t alpha_step = alpha / width;
+
+            for (size_t i = 0; i < width; ++i) {
+                const ssize_t lx = i + x + thumb_size;
+                const ssize_t ly = y + width;
+                const ssize_t lh = thumb_size - (width - i);
+                const argb_t color = base | ARGB_SET_A(alpha - i * alpha_step);
+                pixmap_vline(window, lx, ly, lh, color);
+            }
+            for (size_t i = 0; i < width; ++i) {
+                const ssize_t lx = x + width;
+                const ssize_t ly = y + thumb_size + i;
+                const ssize_t lw = thumb_size - (width - i) + 1;
+                const argb_t color = base | ARGB_SET_A(alpha - i * alpha_step);
+                pixmap_hline(window, lx, ly, lw, color);
+            }
         }
 
-        // frame
-        pixmap_rect(window, x, y, thumb_size, thumb_size, ARGB_SET_A(0xff));
+        // border
+        if (ARGB_GET_A(ctx.clr_border)) {
+            pixmap_rect(window, x, y, thumb_size, thumb_size, ctx.clr_border);
+        }
     }
 }
 
@@ -365,7 +376,7 @@ static bool skip_current(void)
 }
 
 /**
- * Select next item.
+ * Select closest item.
  * @param direction next image position in list
  */
 static void select_nearest(enum action_type direction)
@@ -383,11 +394,13 @@ static void select_nearest(enum action_type direction)
             index = image_list_last();
             ctx.top = image_list_back(index, cols * rows - 1);
             break;
+        case action_prev_file:
         case action_step_left:
             if (index != image_list_first()) {
                 index = image_list_prev_file(index);
             }
             break;
+        case action_next_file:
         case action_step_right:
             if (index != image_list_last()) {
                 index = image_list_next_file(index);
@@ -416,66 +429,77 @@ static void select_nearest(enum action_type direction)
 }
 
 /**
- * Key press handler.
- * @param key code of key pressed
- * @param mods key modifiers (ctrl/alt/shift)
+ * Scroll one page.
+ * @param forward scroll direction
  */
-static void on_keyboard(xkb_keysym_t key, uint8_t mods)
+static void scroll_page(bool forward)
 {
-    const struct keybind* kb = keybind_find(key, mods);
-    if (!kb) {
-        return;
+    const size_t distance =
+        ui_get_width() / ctx.thumb_size + ui_get_height() / ctx.thumb_size;
+    size_t top = ctx.top;
+    size_t index = ctx.selected;
+
+    if (forward) {
+        top = image_list_forward(top, distance);
+        index = image_list_forward(index, distance);
+    } else {
+        top = image_list_back(top, distance);
+        index = image_list_back(index, distance);
     }
 
-    for (size_t i = 0; i < kb->num_actions; ++i) {
-        const struct action* action = &kb->actions[i];
-        switch (action->type) {
-            case action_fullscreen:
-                ui_toggle_fullscreen();
-                break;
-            case action_antialiasing:
-                ctx.thumb_aa = !ctx.thumb_aa;
-                reset_thumbnails();
-                break;
-            case action_first_file:
-            case action_last_file:
-            case action_step_left:
-            case action_step_right:
-            case action_step_up:
-            case action_step_down:
-                select_nearest(action->type);
-                break;
-            case action_skip_file:
-                if (!skip_current()) {
-                    printf("No more images, exit\n");
-                    app_exit(0);
-                    return;
-                }
-                break;
-            case action_reload:
-                reset_thumbnails();
-                break;
-            case action_exec:
-                app_execute(action->params, image_list_get(ctx.selected));
-                break;
-            case action_status:
-                info_update(info_status, "%s", action->params);
-                app_redraw();
-                break;
-            case action_mode:
-                app_switch_mode(ctx.selected);
-                break;
-            case action_info:
-                info_switch(action->params);
-                app_redraw();
-                break;
-            case action_exit:
+    if (index != IMGLIST_INVALID && index != ctx.selected) {
+        ctx.top = top;
+        select_thumbnail(index);
+    }
+}
+
+/**
+ * Apply action.
+ * @param action pointer to the action being performed
+ */
+static void apply_action(const struct action* action)
+{
+    switch (action->type) {
+        case action_antialiasing:
+            ctx.thumb_aa = !ctx.thumb_aa;
+            reset_thumbnails();
+            break;
+        case action_first_file:
+        case action_last_file:
+        case action_prev_file:
+        case action_next_file:
+        case action_step_left:
+        case action_step_right:
+        case action_step_up:
+        case action_step_down:
+            select_nearest(action->type);
+            break;
+        case action_page_up:
+        case action_page_down:
+            scroll_page(action->type == action_page_down);
+            break;
+        case action_skip_file:
+            if (!skip_current()) {
+                printf("No more images, exit\n");
                 app_exit(0);
-                break;
-            default:
-                break;
-        }
-        ++action;
+            }
+            break;
+        case action_reload:
+            reset_thumbnails();
+            fixup_position();
+            break;
+        case action_exec:
+            app_execute(action->params, image_list_get(ctx.selected));
+            break;
+        case action_status:
+            info_update(info_status, "%s", action->params);
+            app_redraw();
+            break;
+        case action_mode:
+            app_switch_mode(ctx.selected);
+            break;
+        default:
+            break;
     }
 }
 
@@ -533,6 +557,14 @@ static enum config_status load_config(const char* key, const char* value)
         if (config_to_color(value, &ctx.clr_select)) {
             status = cfgst_ok;
         }
+    } else if (strcmp(key, "border") == 0) {
+        if (config_to_color(value, &ctx.clr_border)) {
+            status = cfgst_ok;
+        }
+    } else if (strcmp(key, "shadow") == 0) {
+        if (config_to_color(value, &ctx.clr_shadow)) {
+            status = cfgst_ok;
+        }
     } else if (strcmp(key, "antialiasing") == 0) {
         if (config_to_bool(value, &ctx.thumb_aa)) {
             status = cfgst_ok;
@@ -551,6 +583,8 @@ void gallery_create(void)
     ctx.selected = IMGLIST_INVALID;
     ctx.clr_background = 0xff202020;
     ctx.clr_select = 0xff404040;
+    ctx.clr_border = 0xff000000;
+    ctx.clr_shadow = 0xff000000;
 
     // register configuration loader
     config_add_loader("gallery", load_config);
@@ -573,15 +607,12 @@ void gallery_destroy(void)
 void gallery_handle(const struct event* event)
 {
     switch (event->type) {
-        case event_reload:
-            reset_thumbnails();
+        case event_action:
+            apply_action(event->param.action);
             break;
         case event_redraw:
             redraw();
             break;
-        case event_keypress:
-            on_keyboard(event->param.keypress.key, event->param.keypress.mods);
-            break;
         case event_activate:
             select_thumbnail(event->param.activate.index);
             break;
