@@ -5,12 +5,19 @@
 #pragma once
 
 #include "event.h"
+#include "image.h"
 
 /**
  * Create global gallery context.
  */
 void gallery_create(void);
 
+/**
+ * Initialize global gallery context.
+ * @param image initial image to open
+ */
+void gallery_init(struct image* image);
+
 /**
  * Destroy global gallery context.
  */
