@@ -11,6 +11,7 @@ struct image_info;
 
 /** Image context. */
 struct image {
+    size_t index;               ///< Index of the entry in the image list
     char* source;               ///< Image source (e.g. path to the image file)
     const char* name;           ///< Name of the image file
     size_t file_size;           ///< Size of image file
@@ -72,6 +73,14 @@ void image_flip_horizontal(struct image* ctx);
  */
 void image_rotate(struct image* ctx, size_t angle);
 
+/**
+ * Create thumbnail from full size image.
+ * @param image original image
+ * @param size thumbnail size in pixels
+ * @param antialias use antialiasing
+ */
+void image_thumbnail(struct image* image, size_t size, bool antialias);
+
 /**
  * Set image format description.
  * @param ctx image context
