@@ -57,7 +57,6 @@ static const struct keybind_default default_viewer[] = {
     { .key = XKB_KEY_a,           .action = { action_antialiasing, NULL } },
     { .key = XKB_KEY_r,           .action = { action_reload, NULL } },
     { .key = XKB_KEY_i,           .action = { action_info, NULL } },
-    { .key = XKB_KEY_e,           .action = { action_exec, "echo \"Image: %\"" } },
     { .key = XKB_KEY_Return,      .action = { action_mode, NULL } },
     { .key = XKB_KEY_Escape,      .action = { action_exit, NULL } },
     { .key = XKB_KEY_q,           .action = { action_exit, NULL } },
@@ -71,10 +70,12 @@ static const struct keybind_default default_viewer[] = {
     { .key = VKEY_SCROLL_DOWN, .mods = KEYMOD_SHIFT, .action = { action_next_file, NULL } },
     { .key = VKEY_SCROLL_UP,   .mods = KEYMOD_ALT,   .action = { action_prev_frame, NULL } },
     { .key = VKEY_SCROLL_DOWN, .mods = KEYMOD_ALT,   .action = { action_next_frame, NULL } },
+    { .key = XKB_KEY_Delete, .mods = KEYMOD_SHIFT, .action = { action_none, NULL } },
 };
 
 /** Default key bindings for gallery mode. */
 static const struct keybind_default default_gallery[] = {
+    { .key = XKB_KEY_F1,          .action = { action_help, NULL } },
     { .key = XKB_KEY_Home,        .action = { action_first_file, NULL } },
     { .key = XKB_KEY_End,         .action = { action_last_file, NULL } },
     { .key = XKB_KEY_f,           .action = { action_fullscreen, NULL } },
@@ -82,11 +83,12 @@ static const struct keybind_default default_gallery[] = {
     { .key = XKB_KEY_Right,       .action = { action_step_right, NULL } },
     { .key = XKB_KEY_Up,          .action = { action_step_up, NULL } },
     { .key = XKB_KEY_Down,        .action = { action_step_down, NULL } },
+    { .key = XKB_KEY_Prior,       .action = { action_page_up, NULL } },
+    { .key = XKB_KEY_Next,        .action = { action_page_down, NULL } },
     { .key = XKB_KEY_c,           .action = { action_skip_file, NULL } },
     { .key = XKB_KEY_a,           .action = { action_antialiasing, NULL } },
     { .key = XKB_KEY_r,           .action = { action_reload, NULL } },
     { .key = XKB_KEY_i,           .action = { action_info, NULL } },
-    { .key = XKB_KEY_e,           .action = { action_exec, "echo \"Image: %\"" } },
     { .key = XKB_KEY_Return,      .action = { action_mode, NULL } },
     { .key = XKB_KEY_Escape,      .action = { action_exit, NULL } },
     { .key = XKB_KEY_q,           .action = { action_exit, NULL } },
@@ -94,6 +96,7 @@ static const struct keybind_default default_gallery[] = {
     { .key = VKEY_SCROLL_RIGHT,   .action = { action_step_left,  NULL } },
     { .key = VKEY_SCROLL_UP,      .action = { action_step_up,    NULL } },
     { .key = VKEY_SCROLL_DOWN,    .action = { action_step_down,  NULL } },
+    { .key = XKB_KEY_Delete, .mods = KEYMOD_SHIFT, .action = { action_none, NULL } },
 };
 // clang-format on
 
@@ -166,48 +169,43 @@ static bool parse_keymod(const char* name, xkb_keysym_t* key, uint8_t* mods)
  * Allocate new key binding.
  * @param key keyboard key
  * @param mods key modifiers (ctrl/alt/shift)
- * @param action list of actions to add
- * @param sz number of actions in list
+ * @param actions sequence of actions
  */
 static struct keybind* create_binding(xkb_keysym_t key, uint8_t mods,
-                                      const struct action* action, size_t sz)
+                                      struct action_seq* actions)
 {
-    struct keybind* kb;
-
-    if (!action || sz == 0) {
-        return NULL;
-    }
-
-    kb = calloc(1, sizeof(struct keybind) + sizeof(struct action) * sz);
+    struct keybind* kb = calloc(1, sizeof(struct keybind));
     if (!kb) {
         return NULL;
     }
 
     kb->key = key;
     kb->mods = mods;
-
-    // construct actions
-    kb->actions = (struct action*)((uint8_t*)kb + sizeof(struct keybind));
-    kb->num_actions = sz;
-    for (size_t i = 0; i < sz; ++i) {
-        kb->actions[i].type = action[i].type;
-        if (action[i].params) {
-            kb->actions[i].params = str_dup(action[i].params, NULL);
-        }
-    }
+    kb->actions = *actions;
 
     // construct help description
-    if (sz != 1 || kb->actions[0].type != action_none) {
-        char* key_name = keybind_name(key, mods);
+    if (kb->actions.sequence[0].type != action_none) {
+        size_t max_len = 30;
+        char* key_name = keybind_name(kb->key, kb->mods);
         if (key_name) {
-            const struct action* action = &kb->actions[0]; // first only
+            const struct action* action =
+                &kb->actions.sequence[0]; // first only
             str_append(key_name, 0, &kb->help);
-            str_append(": ", 2, &kb->help);
+            str_append(": ", 0, &kb->help);
             str_append(action_typename(action), 0, &kb->help);
             if (action->params) {
-                str_append(" ", 1, &kb->help);
+                str_append(" ", 0, &kb->help);
                 str_append(action->params, 0, &kb->help);
             }
+            if (kb->actions.num > 1) {
+                str_append("; ...", 0, &kb->help);
+            }
+            if (strlen(kb->help) > max_len) {
+                const char* ellipsis = "...";
+                const size_t ellipsis_len = strlen(ellipsis);
+                memcpy(&kb->help[max_len - ellipsis_len], ellipsis,
+                       ellipsis_len + 1);
+            }
             free(key_name);
         }
     }
@@ -222,9 +220,7 @@ static struct keybind* create_binding(xkb_keysym_t key, uint8_t mods,
 static void free_binding(struct keybind* kb)
 {
     if (kb) {
-        for (size_t i = 0; i < kb->num_actions; ++i) {
-            free(kb->actions[i].params);
-        }
+        action_free(&kb->actions);
         free(kb->help);
         free(kb);
     }
@@ -235,11 +231,10 @@ static void free_binding(struct keybind* kb)
  * @param kb head of binding list
  * @param key keyboard key
  * @param mods key modifiers (ctrl/alt/shift)
- * @param action list of actions to add
- * @param sz number of actions in list
+ * @param actions sequence of actions
  */
 static void set_binding(struct keybind** head, xkb_keysym_t key, uint8_t mods,
-                        const struct action* action, size_t sz)
+                        struct action_seq* actions)
 {
     struct keybind* kb;
     struct keybind* prev;
@@ -262,13 +257,53 @@ static void set_binding(struct keybind** head, xkb_keysym_t key, uint8_t mods,
     }
 
     // add to head
-    kb = create_binding(key, mods, action, sz);
-    if (kb) {
+    kb = create_binding(key, mods, actions);
+    if (!kb) {
+        action_free(actions);
+    } else {
         kb->next = *head;
         *head = kb;
     }
 }
 
+/**
+ * Create default binding.
+ * @param head head of binding list
+ * @param kb default key binding description
+ */
+static void set_default(struct keybind** head, const struct keybind_default* kb)
+{
+    struct action_seq actions;
+
+    if (kb->action.type == action_none && kb->key == XKB_KEY_Delete &&
+        kb->mods == KEYMOD_SHIFT) {
+        // special action: Shift+Del
+        actions.num = 2;
+        actions.sequence = malloc(actions.num * sizeof(struct action));
+        if (actions.sequence) {
+            actions.sequence[0].type = action_exec;
+            actions.sequence[0].params = str_dup("rm \"%\"", NULL);
+            actions.sequence[1].type = action_skip_file;
+            actions.sequence[1].params = NULL;
+        }
+    } else {
+        actions.num = 1;
+        actions.sequence = malloc(actions.num * sizeof(struct action));
+        if (actions.sequence) {
+            actions.sequence[0].type = kb->action.type;
+            if (kb->action.params) {
+                actions.sequence[0].params = str_dup(kb->action.params, NULL);
+            } else {
+                actions.sequence[0].params = NULL;
+            }
+        }
+    }
+
+    if (actions.sequence) {
+        set_binding(head, kb->key, kb->mods, &actions);
+    }
+}
+
 /**
  * Load binding from config parameters.
  * @param kb head of binding list
@@ -279,11 +314,9 @@ static void set_binding(struct keybind** head, xkb_keysym_t key, uint8_t mods,
 static enum config_status load_binding(struct keybind** head, const char* key,
                                        const char* value)
 {
+    struct action_seq actions = { 0 };
     xkb_keysym_t keysym;
     uint8_t mods;
-    struct action action[32];
-    struct str_slice action_sl[ARRAY_SIZE(action)];
-    size_t action_num;
 
     // parse keyboard shortcut
     if (!parse_keymod(key, &keysym, &mods)) {
@@ -291,25 +324,11 @@ static enum config_status load_binding(struct keybind** head, const char* key,
     }
 
     // parse actions
-    action_num = str_split(value, ';', action_sl, ARRAY_SIZE(action_sl));
-    if (action_num == 0) {
+    if (!action_create(value, &actions)) {
         return cfgst_invalid_value;
     }
-    for (size_t i = 0; i < action_num; ++i) {
-        struct str_slice* s = &action_sl[i];
-        if (!action_load(&action[i], s->value, s->len)) {
-            while (i--) {
-                free(action[i].params);
-            }
-            return cfgst_invalid_value;
-        }
-    }
 
-    set_binding(head, keysym, mods, action, action_num);
-
-    for (size_t i = 0; i < action_num; ++i) {
-        free(action[i].params);
-    }
+    set_binding(head, keysym, mods, &actions);
 
     return cfgst_ok;
 }
@@ -330,12 +349,10 @@ void keybind_create(void)
 {
     // create default bindings
     for (size_t i = 0; i < ARRAY_SIZE(default_viewer); ++i) {
-        const struct keybind_default* kb = &default_viewer[i];
-        set_binding(&kb_viewer, kb->key, kb->mods, &kb->action, 1);
+        set_default(&kb_viewer, &default_viewer[i]);
     }
     for (size_t i = 0; i < ARRAY_SIZE(default_gallery); ++i) {
-        const struct keybind_default* kb = &default_gallery[i];
-        set_binding(&kb_gallery, kb->key, kb->mods, &kb->action, 1);
+        set_default(&kb_gallery, &default_gallery[i]);
     }
 
     // register configuration loaders
