@@ -30,23 +30,23 @@ struct cmdarg {
 // clang-format off
 static const struct cmdarg arguments[] = {
     { 'g', "gallery",    NULL,    "start in gallery mode",
-                                  GENERAL_CONFIG_SECTION, APP_CFG_GALLERY, "yes" },
+                                  APP_CFG_SECTION, APP_CFG_MODE, APP_MODE_GALLERY },
     { 'r', "recursive",  NULL,    "read directories recursively",
                                   IMGLIST_CFG_SECTION, IMGLIST_CFG_RECURSIVE, "yes" },
     { 'o', "order",      "ORDER", "set sort order for image list: none/[alpha]/random",
                                   IMGLIST_CFG_SECTION, IMGLIST_CFG_ORDER, NULL },
     { 's', "scale",      "SCALE", "set initial image scale: [optimal]/fit/width/height/fill/real",
-                                  GENERAL_CONFIG_SECTION, VIEWER_CFG_SCALE, NULL },
+                                  VIEWER_CFG_SECTION, VIEWER_CFG_SCALE, NULL },
     { 'l', "slideshow",  NULL,    "activate slideshow mode on startup",
-                                  GENERAL_CONFIG_SECTION, VIEWER_CFG_SLIDESHOW, "yes" },
-    { 'f', "fullscreen", NULL,    "show image in full screen mode",
-                                  GENERAL_CONFIG_SECTION, UI_CFG_FULLSCREEN, "yes" },
+                                  VIEWER_CFG_SECTION, VIEWER_CFG_SLIDESHOW, "yes" },
     { 'p', "position",   "POS",   "set window position [parent]/X,Y",
-                                  GENERAL_CONFIG_SECTION, UI_CFG_POSITION, NULL },
-    { 'w', "size",       "SIZE",  "set window size: [parent]/image/W,H",
-                                  GENERAL_CONFIG_SECTION, UI_CFG_SIZE, NULL },
+                                  APP_CFG_SECTION, APP_CFG_POSITION, NULL },
+    { 'w', "size",       "SIZE",  "set window size: fullscreen/[parent]/image/W,H",
+                                  APP_CFG_SECTION, APP_CFG_SIZE, NULL },
+    { 'f', "fullscreen", NULL,    "show image in full screen mode",
+                                  APP_CFG_SECTION, APP_CFG_SIZE, APP_FULLSCREEN },
     { 'a', "class",      "NAME",  "set window class/app_id",
-                                  GENERAL_CONFIG_SECTION, APP_CFG_APP_ID, NULL },
+                                  APP_CFG_SECTION, APP_CFG_APP_ID, NULL },
     { 'c', "config",     "S.K=V", "set configuration parameter: section.key=value",
                                   NULL, NULL, NULL },
     { 'v', "version",    NULL,    "print version info and exit", NULL, NULL, NULL },
@@ -59,8 +59,6 @@ static const struct cmdarg arguments[] = {
  */
 static void print_help(void)
 {
-    char buf_lopt[32];
-
     puts("Usage: " APP_NAME " [OPTION]... [FILE]...");
     puts("Show images from FILE(s).");
     puts("If FILE is -, read standard input.");
@@ -70,12 +68,13 @@ static void print_help(void)
 
     for (size_t i = 0; i < sizeof(arguments) / sizeof(arguments[0]); ++i) {
         const struct cmdarg* arg = &arguments[i];
-        strcpy(buf_lopt, arg->long_opt);
+        char lopt[32];
         if (arg->format) {
-            strcat(buf_lopt, "=");
-            strcat(buf_lopt, arg->format);
+            snprintf(lopt, sizeof(lopt), "%s=%s", arg->long_opt, arg->format);
+        } else {
+            strncpy(lopt, arg->long_opt, sizeof(lopt) - 1);
         }
-        printf("  -%c, --%-14s %s\n", arg->short_opt, buf_lopt, arg->help);
+        printf("  -%c, --%-14s %s\n", arg->short_opt, lopt, arg->help);
     }
 }
 
@@ -131,6 +130,7 @@ static int parse_cmdargs(int argc, char* argv[])
         switch (opt) {
             case 'c':
                 if (!config_command(optarg)) {
+                    fprintf(stderr, "Invalid config: \"%s\"\n", optarg);
                     return -1;
                 }
                 break;
