@@ -13,7 +13,6 @@
 #include "keybind.h"
 #include "loader.h"
 #include "str.h"
-#include "text.h"
 #include "ui.h"
 
 #include <stdio.h>
@@ -65,9 +64,6 @@ struct viewer {
     enum fixed_scale scale_init; ///< Initial scale
     float scale;                 ///< Current scale factor of the image
 
-    struct text_surface* help; ///< Help lines
-    size_t help_sz;            ///< Number of lines in help
-
     bool animation_enable; ///< Animation enable/disable
     int animation_fd;      ///< Animation timer
 
@@ -257,8 +253,8 @@ static void zoom_image(const char* params)
     } else if (str_to_num(params, 0, &percent, 0) && percent != 0 &&
                percent > -1000 && percent < 1000) {
         // zoom in %
-        const double wnd_half_w = ui_get_width() / 2;
-        const double wnd_half_h = ui_get_height() / 2;
+        const double wnd_half_w = (double)ui_get_width() / 2;
+        const double wnd_half_h = (double)ui_get_height() / 2;
         const float step = (ctx.scale / 100) * percent;
         const double center_x = wnd_half_w / ctx.scale - ctx.img_x / ctx.scale;
         const double center_y = wnd_half_h / ctx.scale - ctx.img_y / ctx.scale;
@@ -459,47 +455,6 @@ static void on_slideshow_timer(__attribute__((unused)) void* data)
     slideshow_ctl(next_image(action_next_file));
 }
 
-/**
- * Show/hide help layer.
- */
-static void switch_help(void)
-{
-    const struct keybind* kb;
-
-    app_redraw();
-
-    if (ctx.help) {
-        for (size_t i = 0; i < ctx.help_sz; i++) {
-            free(ctx.help[i].data);
-        }
-        free(ctx.help);
-        ctx.help = NULL;
-        ctx.help_sz = 0;
-        return;
-    }
-
-    // get number of bindings
-    ctx.help_sz = 0;
-    kb = keybind_get();
-    while (kb) {
-        if (kb->help) {
-            ++ctx.help_sz;
-        }
-        kb = kb->next;
-    }
-    ctx.help = calloc(1, ctx.help_sz * sizeof(*ctx.help));
-    if (ctx.help) {
-        size_t i = ctx.help_sz - 1;
-        kb = keybind_get();
-        while (kb) {
-            if (kb->help) {
-                font_render(kb->help, &ctx.help[i--]);
-            }
-            kb = kb->next;
-        }
-    }
-}
-
 /**
  * Draw image.
  * @param wnd pixel map of target window
@@ -564,18 +519,11 @@ static void reload(void)
 static void redraw(void)
 {
     struct pixmap* window = ui_draw_begin();
-    if (!window) {
-        return;
+    if (window) {
+        draw_image(window);
+        info_print(window);
+        ui_draw_commit();
     }
-
-    draw_image(window);
-    info_print(window);
-
-    if (ctx.help) {
-        text_print_lines(window, text_center, ctx.help, ctx.help_sz);
-    }
-
-    ui_draw_commit();
 }
 
 /**
@@ -588,126 +536,86 @@ static void on_resize(void)
 }
 
 /**
- * Key press handler.
- * @param key code of key pressed
- * @param mods key modifiers (ctrl/alt/shift)
+ * Apply action.
+ * @param action pointer to the action being performed
  */
-static void on_keyboard(xkb_keysym_t key, uint8_t mods)
+static void apply_action(const struct action* action)
 {
-    const struct keybind* kb = keybind_find(key, mods);
-
-    if (!kb) {
-        char* name = keybind_name(key, mods);
-        if (name) {
-            info_update(info_status, "Key %s is not bound", name);
-            free(name);
-            app_redraw();
-        }
-        return;
-    }
-
-    // handle actions
-    for (size_t i = 0; i < kb->num_actions; ++i) {
-        const struct action* action = &kb->actions[i];
-        switch (action->type) {
-            case action_none:
-                break;
-            case action_help:
-                switch_help();
-                break;
-            case action_first_file:
-            case action_last_file:
-            case action_prev_dir:
-            case action_next_dir:
-            case action_prev_file:
-            case action_next_file:
-                next_image(action->type);
-                break;
-            case action_skip_file:
-                if (!skip_image()) {
-                    printf("No more images, exit\n");
-                    app_exit(0);
-                    return;
-                }
+    switch (action->type) {
+        case action_first_file:
+        case action_last_file:
+        case action_prev_dir:
+        case action_next_dir:
+        case action_prev_file:
+        case action_next_file:
+            next_image(action->type);
+            break;
+        case action_skip_file:
+            if (skip_image()) {
                 reset_state();
-                break;
-            case action_prev_frame:
-            case action_next_frame:
-                animation_ctl(false);
-                next_frame(action->type == action_next_frame);
-                break;
-            case action_animation:
-                animation_ctl(!ctx.animation_enable);
-                break;
-            case action_slideshow:
-                slideshow_ctl(!ctx.slideshow_enable &&
-                              next_image(action_next_file));
-                break;
-            case action_fullscreen:
-                ui_toggle_fullscreen();
-                break;
-            case action_mode:
-                app_switch_mode(fetcher_current()->index);
-                break;
-            case action_step_left:
-                move_image(true, true, action->params);
-                break;
-            case action_step_right:
-                move_image(true, false, action->params);
-                break;
-            case action_step_up:
-                move_image(false, true, action->params);
-                break;
-            case action_step_down:
-                move_image(false, false, action->params);
-                break;
-            case action_zoom:
-                zoom_image(action->params);
-                break;
-            case action_rotate_left:
-                rotate_image(false);
-                break;
-            case action_rotate_right:
-                rotate_image(true);
-                break;
-            case action_flip_vertical:
-                image_flip_vertical(fetcher_current());
-                app_redraw();
-                break;
-            case action_flip_horizontal:
-                image_flip_horizontal(fetcher_current());
-                app_redraw();
-                break;
-            case action_antialiasing:
-                ctx.antialiasing = !ctx.antialiasing;
-                info_update(info_status, "Anti-aliasing %s",
-                            ctx.antialiasing ? "on" : "off");
-                app_redraw();
-                break;
-            case action_reload:
-                reload();
-                break;
-            case action_info:
-                info_switch(action->params);
-                app_redraw();
-                break;
-            case action_exec:
-                app_execute(action->params, fetcher_current()->source);
-                break;
-            case action_status:
-                info_update(info_status, "%s", action->params);
-                app_redraw();
-                break;
-            case action_exit:
-                if (ctx.help) {
-                    switch_help(); // remove help overlay
-                } else {
-                    app_exit(0);
-                    return;
-                }
-                break;
-        }
-        ++action;
+            } else {
+                printf("No more images, exit\n");
+                app_exit(0);
+            }
+            break;
+        case action_prev_frame:
+        case action_next_frame:
+            animation_ctl(false);
+            next_frame(action->type == action_next_frame);
+            break;
+        case action_animation:
+            animation_ctl(!ctx.animation_enable);
+            break;
+        case action_slideshow:
+            slideshow_ctl(!ctx.slideshow_enable &&
+                          next_image(action_next_file));
+            break;
+        case action_mode:
+            app_switch_mode(fetcher_current()->index);
+            break;
+        case action_step_left:
+            move_image(true, true, action->params);
+            break;
+        case action_step_right:
+            move_image(true, false, action->params);
+            break;
+        case action_step_up:
+            move_image(false, true, action->params);
+            break;
+        case action_step_down:
+            move_image(false, false, action->params);
+            break;
+        case action_zoom:
+            zoom_image(action->params);
+            break;
+        case action_rotate_left:
+            rotate_image(false);
+            break;
+        case action_rotate_right:
+            rotate_image(true);
+            break;
+        case action_flip_vertical:
+            image_flip_vertical(fetcher_current());
+            app_redraw();
+            break;
+        case action_flip_horizontal:
+            image_flip_horizontal(fetcher_current());
+            app_redraw();
+            break;
+        case action_antialiasing:
+            ctx.antialiasing = !ctx.antialiasing;
+            info_update(info_status, "Anti-aliasing %s",
+                        ctx.antialiasing ? "on" : "off");
+            app_redraw();
+            break;
+        case action_reload:
+            reload();
+            break;
+        case action_exec:
+            app_execute(action->params, fetcher_current()->source);
+            break;
+        default:
+            break;
     }
 }
 
@@ -830,9 +738,6 @@ void viewer_destroy(void)
 {
     fetcher_destroy();
 
-    for (size_t i = 0; i < ctx.help_sz; i++) {
-        free(ctx.help[i].data);
-    }
     if (ctx.animation_fd != -1) {
         close(ctx.animation_fd);
     }
@@ -844,8 +749,8 @@ void viewer_destroy(void)
 void viewer_handle(const struct event* event)
 {
     switch (event->type) {
-        case event_reload:
-            reload();
+        case event_action:
+            apply_action(event->param.action);
             break;
         case event_redraw:
             redraw();
@@ -853,9 +758,6 @@ void viewer_handle(const struct event* event)
         case event_resize:
             on_resize();
             break;
-        case event_keypress:
-            on_keyboard(event->param.keypress.key, event->param.keypress.mods);
-            break;
         case event_drag:
             on_drag(event->param.drag.dx, event->param.drag.dy);
             break;
