@@ -35,8 +35,8 @@ TEST_F(Keybind, AddOne)
     ASSERT_NE(kb, nullptr);
     ASSERT_EQ(kb->key, static_cast<xkb_keysym_t>('a'));
     ASSERT_EQ(kb->mods, static_cast<uint8_t>(0));
-    ASSERT_EQ(kb->num_actions, static_cast<size_t>(1));
-    ASSERT_EQ(kb->actions[0].type, action_exit);
+    ASSERT_EQ(kb->actions.num, static_cast<size_t>(1));
+    ASSERT_EQ(kb->actions.sequence[0].type, action_exit);
     ASSERT_STREQ(kb->help, "a: exit");
     ASSERT_EQ(kb->next, nullptr);
 }
@@ -50,9 +50,9 @@ TEST_F(Keybind, Replace)
     const struct keybind* kb = keybind_get();
     ASSERT_NE(kb, nullptr);
     ASSERT_EQ(kb->key, static_cast<xkb_keysym_t>('b'));
-    ASSERT_EQ(kb->actions[0].type, action_reload);
+    ASSERT_EQ(kb->actions.sequence[0].type, action_reload);
     ASSERT_EQ(kb->next->key, static_cast<xkb_keysym_t>('a'));
-    ASSERT_EQ(kb->next->actions[0].type, action_exit);
+    ASSERT_EQ(kb->next->actions.sequence[0].type, action_exit);
     ASSERT_EQ(kb->next->next, nullptr);
 }
 
@@ -67,14 +67,14 @@ TEST_F(Keybind, Find)
     ASSERT_NE(kb, nullptr);
     ASSERT_EQ(kb->key, static_cast<xkb_keysym_t>('a'));
     ASSERT_EQ(kb->mods, static_cast<uint8_t>(0));
-    ASSERT_EQ(kb->actions[0].type, action_exit);
+    ASSERT_EQ(kb->actions.sequence[0].type, action_exit);
 
     kb = keybind_find('b', KEYMOD_CTRL | KEYMOD_ALT | KEYMOD_SHIFT);
     ASSERT_NE(kb, nullptr);
     ASSERT_EQ(kb->key, static_cast<xkb_keysym_t>('b'));
     ASSERT_EQ(kb->mods,
               static_cast<uint8_t>(KEYMOD_CTRL | KEYMOD_ALT | KEYMOD_SHIFT));
-    ASSERT_EQ(kb->actions[0].type, action_reload);
+    ASSERT_EQ(kb->actions.sequence[0].type, action_reload);
 
     ASSERT_EQ(keybind_find('a', KEYMOD_CTRL), nullptr);
     ASSERT_EQ(keybind_find('c', 0), nullptr);
@@ -106,8 +106,8 @@ TEST_F(Keybind, ActionParams)
     ASSERT_NE(kb, nullptr);
     ASSERT_EQ(kb->key, static_cast<xkb_keysym_t>('a'));
     ASSERT_EQ(kb->mods, static_cast<uint8_t>(0));
-    ASSERT_EQ(kb->num_actions, static_cast<size_t>(1));
-    ASSERT_EQ(kb->actions[0].type, action_exit);
+    ASSERT_EQ(kb->actions.num, static_cast<size_t>(1));
+    ASSERT_EQ(kb->actions.sequence[0].type, action_exit);
     ASSERT_EQ(kb->next, nullptr);
 
     ASSERT_EQ(config_set(section, "a", "status  \t params 1 2 3\t"), cfgst_ok);
@@ -115,24 +115,24 @@ TEST_F(Keybind, ActionParams)
     ASSERT_NE(kb, nullptr);
     ASSERT_EQ(kb->key, static_cast<xkb_keysym_t>('a'));
     ASSERT_EQ(kb->mods, static_cast<uint8_t>(0));
-    ASSERT_EQ(kb->num_actions, static_cast<size_t>(1));
-    ASSERT_EQ(kb->actions[0].type, action_status);
-    ASSERT_STREQ(kb->actions[0].params, "params 1 2 3");
+    ASSERT_EQ(kb->actions.num, static_cast<size_t>(1));
+    ASSERT_EQ(kb->actions.sequence[0].type, action_status);
+    ASSERT_STREQ(kb->actions.sequence[0].params, "params 1 2 3");
     ASSERT_EQ(kb->next, nullptr);
 }
 
 TEST_F(Keybind, Multiaction)
 {
-    ASSERT_EQ(config_set(section, "a", "exec cmd;\nreload;\t exit"), cfgst_ok);
+    ASSERT_EQ(config_set(section, "a", "exec cmd;reload;exit"), cfgst_ok);
 
     const struct keybind* kb = keybind_get();
     ASSERT_NE(kb, nullptr);
     ASSERT_EQ(kb->key, static_cast<xkb_keysym_t>('a'));
     ASSERT_EQ(kb->mods, static_cast<uint8_t>(0));
-    ASSERT_EQ(kb->num_actions, static_cast<size_t>(3));
-    ASSERT_EQ(kb->actions[0].type, action_exec);
-    ASSERT_EQ(kb->actions[1].type, action_reload);
-    ASSERT_EQ(kb->actions[2].type, action_exit);
-    ASSERT_STREQ(kb->actions[0].params, "cmd");
+    ASSERT_EQ(kb->actions.num, static_cast<size_t>(3));
+    ASSERT_EQ(kb->actions.sequence[0].type, action_exec);
+    ASSERT_EQ(kb->actions.sequence[1].type, action_reload);
+    ASSERT_EQ(kb->actions.sequence[2].type, action_exit);
+    ASSERT_STREQ(kb->actions.sequence[0].params, "cmd");
     ASSERT_EQ(kb->next, nullptr);
 }
