@@ -13,13 +13,18 @@ extern "C" {
 
 // stubs for linker (application and ui are not included to tests)
 extern "C" {
-void app_watch(int, fd_callback) { }
+void app_watch(int, fd_callback, void*) { }
 void app_reload() { }
 void app_redraw() { }
 void app_on_resize() { }
 void app_on_keyboard(xkb_keysym_t, uint8_t) { }
 void app_on_drag(int, int) { }
 void app_exit(int) { }
+void app_on_load(struct image*, size_t) { }
+bool app_is_viewer()
+{
+    return true;
+}
 }
 
 class Loader : public ::testing::Test {
@@ -28,21 +33,20 @@ protected:
 
     void Load(const char* file)
     {
-        enum loader_status status;
-        image = loader_load_image(file, &status);
+        EXPECT_EQ(loader_from_source(file, &image), ldr_success);
         ASSERT_NE(image, nullptr);
-        EXPECT_EQ(status, ldr_success);
-        EXPECT_NE(image->frames[0].pm.width, 0);
-        EXPECT_NE(image->frames[0].pm.height, 0);
-        EXPECT_NE(image->frames[0].pm.data[0], 0);
+        EXPECT_NE(image->frames[0].pm.width, static_cast<size_t>(0));
+        EXPECT_NE(image->frames[0].pm.height, static_cast<size_t>(0));
+        EXPECT_NE(image->frames[0].pm.data[0], static_cast<argb_t>(0));
     }
     struct image* image = nullptr;
 };
 
 TEST_F(Loader, External)
 {
-    image =
-        loader_load_image(LDRSRC_EXEC "cat " TEST_DATA_DIR "/image.bmp", NULL);
+    const enum loader_status status = loader_from_source(
+        LDRSRC_EXEC "cat " TEST_DATA_DIR "/image.bmp", &image);
+    EXPECT_EQ(status, ldr_success);
     ASSERT_NE(image, nullptr);
 }
 
